Class: Writexlsx::Chart::Scatter
- Inherits:
-
Writexlsx::Chart
- Object
- Writexlsx::Chart
- Writexlsx::Chart::Scatter
- Includes:
- Utility, WriteDPtPoint
- Defined in:
- lib/write_xlsx/chart/scatter.rb
Overview
The Scatter chart module also supports the following sub-types:
markers_only (the default)
straight_with_markers
straight
smooth_with_markers
smooth
These can be specified at creation time via the add_chart() Worksheet method:
chart = workbook.add_chart(
:type => 'scatter',
:subtype => 'straight_with_markers'
)
Constant Summary
Constants included from Utility
Utility::COL_MAX, Utility::ROW_MAX, Utility::SHEETNAME_MAX, Utility::STR_MAX
Instance Attribute Summary
Attributes inherited from Writexlsx::Chart
#already_inserted, #axis2_ids, #combined, #date_category, #embedded, #formula_data, #formula_ids, #height, #id, #index, #label_position_default, #label_positions, #name, #palette, #protection, #series_index, #width, #writer, #x2_axis, #x_offset, #x_scale, #y2_axis, #y_offset, #y_scale
Instance Method Summary collapse
-
#combine(chart) ⇒ Object
Override parent method to add a warning.
-
#initialize(subtype) ⇒ Scatter
constructor
A new instance of Scatter.
-
#modify_series_formatting ⇒ Object
Add default formatting to the series data unless it has already been specified by the user.
-
#write_cat_val_axis(x_axis, y_axis, axis_ids, position) ⇒ Object
Write the <c:valAx> element.
-
#write_chart_type(params) ⇒ Object
Override the virtual superclass method with a chart specific method.
-
#write_plot_area ⇒ Object
Over-ridden to have 2 valAx elements for scatter charts instead of catAx/valAx.
-
#write_scatter_chart(params) ⇒ Object
Write the <c:scatterChart> element.
-
#write_scatter_style(val) ⇒ Object
Write the <c:scatterStyle> element.
-
#write_ser(series) ⇒ Object
Over-ridden to write c:xVal/c:yVal instead of c:cat/c:val elements.
-
#write_x_val(series) ⇒ Object
Write the <c:xVal> element.
-
#write_y_val(series) ⇒ Object
Write the <c:yVal> element.
Methods included from WriteDPtPoint
Methods included from Utility
#absolute_char, #check_dimensions, #check_dimensions_and_update_max_min_values, #check_parameter, #convert_date_time, #dash_types, delete_files, #fill_properties, #float_to_str, #layout_properties, #line_fill_properties, #line_properties, #palette_color, #pixels_to_points, #process_workbook_options, #ptrue?, #put_deprecate_message, #quote_sheetname, #r_id_attributes, #row_col_notation, #shape_style_base, #store_col_max_min_values, #store_row_max_min_values, #substitute_cellref, #underline_attributes, #v_shape_attributes_base, #v_shape_style_base, #value_or_raise, #write_anchor, #write_auto_fill, #write_color, #write_comment_path, #write_div, #write_fill, #write_font, #write_stroke, #write_xml_declaration, #xl_cell_to_rowcol, #xl_col_to_name, #xl_range, #xl_range_formula, #xl_rowcol_to_cell, #xml_str
Methods inherited from Writexlsx::Chart
#add_series, #already_inserted?, #assemble_xml_file, #convert_font_args, #data_id, factory, #is_secondary?, #params_to_font, #process_names, #set_chartarea, #set_drop_lines, #set_embedded_config_data, #set_high_low_lines, #set_legend, #set_plotarea, #set_size, #set_style, #set_table, #set_title, #set_up_down_bars, #set_x2_axis, #set_x_axis, #set_xml_writer, #set_y2_axis, #set_y_axis, #show_blanks_as, #show_hidden_data, #write_bar_chart, #write_val_axis
Constructor Details
#initialize(subtype) ⇒ Scatter
Returns a new instance of Scatter.
39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 |
# File 'lib/write_xlsx/chart/scatter.rb', line 39 def initialize(subtype) super(subtype) @subtype = subtype || 'marker_only' @cross_between = 'midCat' @horiz_val_axis = 0 @val_axis_position = 'b' @smooth_allowed = 1 # Set the available data label positions for this chart type. @label_position_default = 'right' @label_positions = { 'center' => 'ctr', 'right' => 'r', 'left' => 'l', 'above' => 't', 'below' => 'b', # For backward compatibility. 'top' => 't', 'bottom' => 'b' } end |
Instance Method Details
#combine(chart) ⇒ Object
Override parent method to add a warning.
64 65 66 |
# File 'lib/write_xlsx/chart/scatter.rb', line 64 def combine(chart) raise 'Combined chart not currently supported with scatter chart as the primary chart' end |
#modify_series_formatting ⇒ Object
Add default formatting to the series data unless it has already been specified by the user.
212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 |
# File 'lib/write_xlsx/chart/scatter.rb', line 212 def modify_series_formatting # The default scatter style "markers only" requires a line type if @subtype == 'marker_only' # Go through each series and define default values. @series.each do |series| # Set a line type unless there is already a user defined type. unless series.line_defined? series.line = line_properties(:width => 2.25, :none => 1, :_defined => 1) end end end # Turn markers off for subtypes that don't have them unless @subtype =~ /marker/ # Go through each series and define default values. @series.each do |series| # Set a marker type unless there is already a user defined type. unless ptrue?(series.marker) series.marker = Marker.new(:type => 'none', :_defined => 1) end end end end |
#write_cat_val_axis(x_axis, y_axis, axis_ids, position) ⇒ Object
Write the <c:valAx> element. This is for the second valAx in scatter plots.
Usually the X axis.
242 243 244 245 246 247 248 249 250 251 |
# File 'lib/write_xlsx/chart/scatter.rb', line 242 def write_cat_val_axis(x_axis, y_axis, axis_ids, position) # :nodoc: return unless axis_ids && !axis_ids.empty? write_val_axis_base( y_axis, x_axis, axis_ids[1], axis_ids[0], x_axis.position || position || @val_axis_position ) end |
#write_chart_type(params) ⇒ Object
Override the virtual superclass method with a chart specific method.
71 72 73 74 |
# File 'lib/write_xlsx/chart/scatter.rb', line 71 def write_chart_type(params) # Write the c:areaChart element. write_scatter_chart(params) end |
#write_plot_area ⇒ Object
Over-ridden to have 2 valAx elements for scatter charts instead of catAx/valAx.
Write the <c:plotArea> element.
134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 |
# File 'lib/write_xlsx/chart/scatter.rb', line 134 def write_plot_area @writer.tag_elements('c:plotArea') do # Write the c:layout element. write_layout(@plotarea.layout, 'plot') # Write the subclass chart type elements for primary and secondary axes. write_chart_type(:primary_axes => 1) write_chart_type(:primary_axes => 0) # Write c:catAx and c:valAx elements for series using primary axes. write_cat_val_axis(@x_axis, @y_axis, @axis_ids, 'b') tmp = @horiz_val_axis @horiz_val_axis = 1 write_val_axis(@x_axis, @y_axis, @axis_ids, 'l') @horiz_val_axis = tmp # Write c:valAx and c:catAx elements for series using secondary axes. write_cat_val_axis(@x2_axis, @y2_axis, @axis2_ids, 'b') @horiz_val_axis = 1 write_val_axis(@x2_axis, @y2_axis, @axis2_ids, 'l') # Write the c:spPr element for the plotarea formatting. write_sp_pr(@plotarea) end end |
#write_scatter_chart(params) ⇒ Object
Write the <c:scatterChart> element.
79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 |
# File 'lib/write_xlsx/chart/scatter.rb', line 79 def write_scatter_chart(params) if params[:primary_axes] == 1 series = get_primary_axes_series else series = get_secondary_axes_series end return if series.empty? style = 'lineMarker' # Set the user defined chart subtype style = 'smoothMarker' if ['smooth_with_markers', 'smooth'].include?(@subtype) # Add default formatting to the series data. modify_series_formatting @writer.tag_elements('c:scatterChart') do # Write the c:scatterStyle element. write_scatter_style(style) # Write the series elements. series.each {|s| write_series(s)} # Write the c:marker element. write_marker_value # Write the c:axId elements write_axis_ids(params) end end |
#write_scatter_style(val) ⇒ Object
Write the <c:scatterStyle> element.
204 205 206 |
# File 'lib/write_xlsx/chart/scatter.rb', line 204 def write_scatter_style(val) @writer.empty_tag('c:scatterStyle', [ ['val', val] ]) end |
#write_ser(series) ⇒ Object
Over-ridden to write c:xVal/c:yVal instead of c:cat/c:val elements.
Write the <c:ser> element.
111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 |
# File 'lib/write_xlsx/chart/scatter.rb', line 111 def write_ser(series) @writer.tag_elements('c:ser') do write_ser_base(series) # Write the c:xVal element. write_x_val(series) # Write the c:yVal element. write_y_val(series) # Write the c:smooth element. if @subtype =~ /smooth/ && !series.smooth write_c_smooth(1) else write_c_smooth(series.smooth) end end @series_index += 1 end |
#write_x_val(series) ⇒ Object
Write the <c:xVal> element.
165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 |
# File 'lib/write_xlsx/chart/scatter.rb', line 165 def write_x_val(series) formula = series.categories data_id = series.cat_data_id data = @formula_data[data_id] @writer.tag_elements('c:xVal') do # Check the type of cached data. type = get_data_type(data) # TODO. Can a scatter plot have non-numeric data. if type == 'str' # Write the c:numRef element. write_str_ref(formula, data, type) else write_num_ref(formula, data, type) end end end |
#write_y_val(series) ⇒ Object
Write the <c:yVal> element.
188 189 190 191 192 193 194 195 196 197 198 199 |
# File 'lib/write_xlsx/chart/scatter.rb', line 188 def write_y_val(series) formula = series.values data_id = series.val_data_id data = @formula_data[data_id] @writer.tag_elements('c:yVal') do # Unlike Cat axes data should only be numeric # Write the c:numRef element. write_num_ref(formula, data, 'num') end end |