Module: Writexlsx::Worksheet::CommentsSupport
- Included in:
- Writexlsx::Worksheet
- Defined in:
- lib/write_xlsx/worksheet/comments_support.rb
Instance Method Summary collapse
-
#buttons_data ⇒ Object
:nodoc:.
-
#comments_author=(author) ⇒ Object
This method is used to set the default author of all cell comments.
-
#comments_visible? ⇒ Boolean
:nodoc:.
-
#has_comments? ⇒ Boolean
:nodoc:.
-
#has_header_vml? ⇒ Boolean
:nodoc:.
-
#has_vml? ⇒ Boolean
:nodoc:.
-
#header_images_data ⇒ Object
:nodoc:.
- #num_comments_block ⇒ Object
-
#set_comments_author(author) ⇒ Object
This method is deprecated.
-
#show_comments(visible = true) ⇒ Object
This method is used to make all cell comments visible when a worksheet is opened.
-
#sorted_comments ⇒ Object
:nodoc:.
Instance Method Details
#buttons_data ⇒ Object
:nodoc:
52 53 54 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 52 def # :nodoc: @buttons_array end |
#comments_author=(author) ⇒ Object
This method is used to set the default author of all cell comments.
22 23 24 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 22 def () @comments_author = || '' end |
#comments_visible? ⇒ Boolean
:nodoc:
15 16 17 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 15 def comments_visible? # :nodoc: !!@comments_visible end |
#has_comments? ⇒ Boolean
:nodoc:
40 41 42 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 40 def has_comments? # :nodoc: !@comments.empty? end |
#has_header_vml? ⇒ Boolean
:nodoc:
48 49 50 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 48 def has_header_vml? # :nodoc: !(header_images.empty? && .empty?) end |
#has_vml? ⇒ Boolean
:nodoc:
44 45 46 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 44 def has_vml? # :nodoc: @has_vml end |
#header_images_data ⇒ Object
:nodoc:
56 57 58 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 56 def header_images_data # :nodoc: @header_images_array end |
#num_comments_block ⇒ Object
36 37 38 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 36 def num_comments_block @comments.size / 1024 end |
#set_comments_author(author) ⇒ Object
This method is deprecated. use comments_author=().
27 28 29 30 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 27 def () ("#{self}.set_comments_author") self. = end |
#show_comments(visible = true) ⇒ Object
This method is used to make all cell comments visible when a worksheet is opened.
11 12 13 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 11 def show_comments(visible = true) @comments_visible = visible end |
#sorted_comments ⇒ Object
:nodoc:
32 33 34 |
# File 'lib/write_xlsx/worksheet/comments_support.rb', line 32 def sorted_comments # :nodoc: @comments.sorted_comments end |